clash-protocols
Safe HaskellNone
LanguageGHC2021

Protocols.Experimental.Axi4.Common

Description

Shared types and helpers for the experimental AXI4-family channel definitions.

Synopsis

Documentation

type BurstType (keep :: Bool) = KeepType keep BurstMode Source #

Enables or disables BurstMode

type BurstLengthType (keep :: Bool) = KeepType keep (Index (2 ^ 8)) Source #

Enables or disables burst length

type AwCacheType (keep :: Bool) = KeepType keep AwCache Source #

Enables or disables the AwCache for Write Address operations

type ArCacheType (keep :: Bool) = KeepType keep ArCache Source #

Enables or disables the ArCache for Read Address operations

type LastType (keep :: Bool) = KeepType keep Bool Source #

Enables or disables a boolean indicating whether a transaction is done

type LockType (keep :: Bool) = KeepType keep AtomicAccess Source #

Enables or disables AtomicAccess

type PermissionsType (keep :: Bool) = KeepType keep Permissions Source #

Enables or disables Permissions

type QosType (keep :: Bool) = KeepType keep Qos Source #

Enables or disables Qos

type RegionType (keep :: Bool) = KeepType keep (BitVector 4) Source #

Enables or disables region type

type ResponseType (keep :: Bool) = KeepType keep Resp Source #

Enables or disables Resp

type SizeType (keep :: Bool) = KeepType keep BurstSize Source #

Enables or disables BurstSize

type StrictStrobeType (byteSize :: Nat) (keepStrobe :: Bool) = Vec byteSize (StrobeDataType keepStrobe) Source #

byteSize bytes of data, with keepStrobe determining whether to include a strobe value or not.

type family StrobeDataType (keepStrobe :: Bool) = (t :: Type) | t -> keepStrobe where ... Source #

Enable or disable a strobe value.

class KeepStrobeClass (keepStrobe :: Bool) where Source #

We want to define operations on StrobeDataType that work for both possibilities (keepStrobe = 'True and keepStrobe = 'False), but we can't pattern match directly. Instead we need to define a class and instantiate the class for both 'True and 'False.

Methods

getKeepStrobe :: StrobeDataType keepStrobe -> Bool Source #

Get the value of keepStrobe at the term level.

toStrobeDataType :: Bool -> BitVector 8 -> StrobeDataType keepStrobe Source #

Convert a byte into a possibly-strobed byte. The Bool value determines the strobe value if strobe is enabled.

fromStrobeDataType :: StrobeDataType keepStrobe -> Maybe (BitVector 8) Source #

Convert a possibly-strobed byte into a byte, or Nothing if strobe is enabled and strobe = false.

type Qos = Index ((2 ^ 4) - 1) Source #

The protocol does not specify the exact use of the QoS identifier. This AXI specification recommends that AxQOS is used as a priority indicator for the associated write or read transaction. A higher value indicates a higher priority transaction.

A default value of 0 indicates that the interface is not participating in any QoS scheme.

data BurstMode Source #

The burst type and the size information, determine how the address for each transfer within the burst is calculated.

Constructors

BmFixed

In a fixed burst, the address is the same for every transfer in the burst. This burst type is used for repeated accesses to the same location such as when loading or emptying a FIFO

BmIncr

Incrementing. In an incrementing burst, the address for each transfer in the burst is an increment of the address for the previous transfer. The increment value depends on the size of the transfer. For example, the address for each transfer in a burst with a size of four bytes is the previous address plus four. This burst type is used for accesses to normal sequential memory.

BmWrap

A wrapping burst is similar to an incrementing burst, except that the address wraps around to a lower address if an upper address limit is reached. The following restrictions apply to wrapping bursts:

  • the start address must be aligned to the size of each transfer
  • the length of the burst must be 2, 4, 8, or 16 transfers.

The behavior of a wrapping burst is:

  • The lowest address used by the burst is aligned to the total size of the data to be transferred, that is, to ((size of each transfer in the burst) × (number of transfers in the burst)). This address is defined as the _wrap boundary_.
  • After each transfer, the address increments in the same way as for an INCR burst. However, if this incremented address is ((wrap boundary) + (total size of data to be transferred)) then the address wraps round to the wrap boundary.
  • The first transfer in the burst can use an address that is higher than the wrap boundary, subject to the restrictions that apply to wrapping bursts. This means that the address wraps for any WRAP burst for which the first address is higher than the wrap boundary.

This burst type is used for cache line accesses.

Instances

Instances details
BitPack BurstMode Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX BurstMode Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX BurstMode Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFData BurstMode Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: BurstMode -> () #

Generic BurstMode Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep BurstMode 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep BurstMode = D1 ('MetaData "BurstMode" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "BmFixed" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "BmIncr" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "BmWrap" 'PrefixI 'False) (U1 :: Type -> Type)))
Show BurstMode Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Eq BurstMode Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type BitSize BurstMode Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep BurstMode Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep BurstMode = D1 ('MetaData "BurstMode" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "BmFixed" 'PrefixI 'False) (U1 :: Type -> Type) :+: (C1 ('MetaCons "BmIncr" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "BmWrap" 'PrefixI 'False) (U1 :: Type -> Type)))

data BurstSize Source #

The maximum number of bytes to transfer in each data transfer, or beat, in a burst.

Constructors

Bs1 
Bs2 
Bs4 
Bs8 
Bs16 
Bs32 
Bs64 
Bs128 

Instances

Instances details
BitPack BurstSize Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX BurstSize Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX BurstSize Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFData BurstSize Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: BurstSize -> () #

Generic BurstSize Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep BurstSize 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep BurstSize = D1 ('MetaData "BurstSize" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (((C1 ('MetaCons "Bs1" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Bs2" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "Bs4" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Bs8" 'PrefixI 'False) (U1 :: Type -> Type))) :+: ((C1 ('MetaCons "Bs16" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Bs32" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "Bs64" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Bs128" 'PrefixI 'False) (U1 :: Type -> Type))))
Show BurstSize Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Eq BurstSize Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type BitSize BurstSize Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep BurstSize Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep BurstSize = D1 ('MetaData "BurstSize" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (((C1 ('MetaCons "Bs1" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Bs2" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "Bs4" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Bs8" 'PrefixI 'False) (U1 :: Type -> Type))) :+: ((C1 ('MetaCons "Bs16" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Bs32" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "Bs64" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Bs128" 'PrefixI 'False) (U1 :: Type -> Type))))

burstSizeToNum :: Num a => BurstSize -> a Source #

Convert burst size to a numeric value

data Bufferable Source #

Whether a transaction is bufferable

Constructors

NonBufferable 
Bufferable 

Instances

Instances details
BitPack Bufferable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX Bufferable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX Bufferable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFData Bufferable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: Bufferable -> () #

Generic Bufferable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep Bufferable 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Bufferable = D1 ('MetaData "Bufferable" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "NonBufferable" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Bufferable" 'PrefixI 'False) (U1 :: Type -> Type))
Show Bufferable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Eq Bufferable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type BitSize Bufferable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Bufferable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Bufferable = D1 ('MetaData "Bufferable" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "NonBufferable" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Bufferable" 'PrefixI 'False) (U1 :: Type -> Type))

data Allocate Source #

When set to LookupCache, it is recommended that this transaction is allocated in the cache for performance reasons.

Constructors

NoLookupCache 
LookupCache 

Instances

Instances details
BitPack Allocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX Allocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX Allocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFData Allocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: Allocate -> () #

Generic Allocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep Allocate 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Allocate = D1 ('MetaData "Allocate" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "NoLookupCache" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "LookupCache" 'PrefixI 'False) (U1 :: Type -> Type))

Methods

from :: Allocate -> Rep Allocate x #

to :: Rep Allocate x -> Allocate #

Show Allocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Eq Allocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type BitSize Allocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Allocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Allocate = D1 ('MetaData "Allocate" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "NoLookupCache" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "LookupCache" 'PrefixI 'False) (U1 :: Type -> Type))

data OtherAllocate Source #

When set to OtherLookupCache, it is recommended that this transaction is allocated in the cache for performance reasons.

Instances

Instances details
BitPack OtherAllocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX OtherAllocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX OtherAllocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFData OtherAllocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: OtherAllocate -> () #

Generic OtherAllocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep OtherAllocate 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep OtherAllocate = D1 ('MetaData "OtherAllocate" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "OtherNoLookupCache" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "OtherLookupCache" 'PrefixI 'False) (U1 :: Type -> Type))
Show OtherAllocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Eq OtherAllocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type BitSize OtherAllocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep OtherAllocate Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep OtherAllocate = D1 ('MetaData "OtherAllocate" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "OtherNoLookupCache" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "OtherLookupCache" 'PrefixI 'False) (U1 :: Type -> Type))

type AwCache = (Bufferable, Modifiable, OtherAllocate, Allocate) Source #

Memory attributes. Note that the Allocate and OtherAllocate bits are in different positions for read and write requests.

type ArCache = (Bufferable, Modifiable, Allocate, OtherAllocate) Source #

Memory attributes. Note that the Allocate and OtherAllocate bits are in different positions for read and write requests.

data Resp Source #

Status of the write transaction.

Constructors

ROkay

Normal access success. Indicates that a normal access has been successful. Can also indicate an exclusive access has failed.

RExclusiveOkay

Exclusive access okay. Indicates that either the read or write portion of an exclusive access has been successful.

RSlaveError

Slave error. Used when the access has reached the slave successfully, but the slave wishes to return an error condition to the originating master.

RDecodeError

Decode error. Generated, typically by an interconnect component, to indicate that there is no slave at the transaction address.

Instances

Instances details
BitPack Resp Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type BitSize Resp 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX Resp Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX Resp Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

showsPrecX :: Int -> Resp -> ShowS #

showX :: Resp -> String #

showListX :: [Resp] -> ShowS #

NFData Resp Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: Resp -> () #

Generic Resp Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep Resp 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Resp = D1 ('MetaData "Resp" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) ((C1 ('MetaCons "ROkay" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "RExclusiveOkay" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "RSlaveError" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "RDecodeError" 'PrefixI 'False) (U1 :: Type -> Type)))

Methods

from :: Resp -> Rep Resp x #

to :: Rep Resp x -> Resp #

Show Resp Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

showsPrec :: Int -> Resp -> ShowS #

show :: Resp -> String #

showList :: [Resp] -> ShowS #

Eq Resp Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

(==) :: Resp -> Resp -> Bool #

(/=) :: Resp -> Resp -> Bool #

type BitSize Resp Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Resp Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Resp = D1 ('MetaData "Resp" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) ((C1 ('MetaCons "ROkay" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "RExclusiveOkay" 'PrefixI 'False) (U1 :: Type -> Type)) :+: (C1 ('MetaCons "RSlaveError" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "RDecodeError" 'PrefixI 'False) (U1 :: Type -> Type)))

data AtomicAccess Source #

Whether a resource is accessed with exclusive access or not

Instances

Instances details
BitPack AtomicAccess Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX AtomicAccess Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX AtomicAccess Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFData AtomicAccess Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: AtomicAccess -> () #

Generic AtomicAccess Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep AtomicAccess 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep AtomicAccess = D1 ('MetaData "AtomicAccess" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "NonExclusiveAccess" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "ExclusiveAccess" 'PrefixI 'False) (U1 :: Type -> Type))
Show AtomicAccess Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Eq AtomicAccess Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type BitSize AtomicAccess Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep AtomicAccess Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep AtomicAccess = D1 ('MetaData "AtomicAccess" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "NonExclusiveAccess" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "ExclusiveAccess" 'PrefixI 'False) (U1 :: Type -> Type))

data Modifiable Source #

Whether transaction can be modified

Constructors

Modifiable 
NonModifiable 

Instances

Instances details
BitPack Modifiable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX Modifiable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX Modifiable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFData Modifiable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: Modifiable -> () #

Generic Modifiable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep Modifiable 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Modifiable = D1 ('MetaData "Modifiable" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "Modifiable" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "NonModifiable" 'PrefixI 'False) (U1 :: Type -> Type))
Show Modifiable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Eq Modifiable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type BitSize Modifiable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Modifiable Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Modifiable = D1 ('MetaData "Modifiable" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "Modifiable" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "NonModifiable" 'PrefixI 'False) (U1 :: Type -> Type))

data Secure Source #

An AXI master might support Secure and Non-secure operating states, and extend this concept of security to memory access.

Constructors

Secure 
NonSecure 

Instances

Instances details
BitPack Secure Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type BitSize Secure 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX Secure Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX Secure Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFData Secure Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: Secure -> () #

Generic Secure Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep Secure 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Secure = D1 ('MetaData "Secure" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "Secure" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "NonSecure" 'PrefixI 'False) (U1 :: Type -> Type))

Methods

from :: Secure -> Rep Secure x #

to :: Rep Secure x -> Secure #

Show Secure Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Eq Secure Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

(==) :: Secure -> Secure -> Bool #

(/=) :: Secure -> Secure -> Bool #

type BitSize Secure Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Secure Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Secure = D1 ('MetaData "Secure" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "Secure" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "NonSecure" 'PrefixI 'False) (U1 :: Type -> Type))

data Privileged Source #

An AXI master might support more than one level of operating privilege, and extend this concept of privilege to memory access.

Constructors

NotPrivileged 
Privileged 

Instances

Instances details
BitPack Privileged Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX Privileged Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX Privileged Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFData Privileged Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: Privileged -> () #

Generic Privileged Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep Privileged 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Privileged = D1 ('MetaData "Privileged" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "NotPrivileged" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Privileged" 'PrefixI 'False) (U1 :: Type -> Type))
Show Privileged Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Eq Privileged Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type BitSize Privileged Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Privileged Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep Privileged = D1 ('MetaData "Privileged" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "NotPrivileged" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Privileged" 'PrefixI 'False) (U1 :: Type -> Type))

data InstructionOrData Source #

Whether the transaction is an instruction access or a data access. The AXI protocol defines this indication as a hint. It is not accurate in all cases, for example, where a transaction contains a mix of instruction and data items. This specification recommends that a master sets it to Data, to indicate a data access unless the access is specifically known to be an instruction access.

Constructors

Data 
Instruction 

Instances

Instances details
BitPack InstructionOrData Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFDataX InstructionOrData Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

ShowX InstructionOrData Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

NFData InstructionOrData Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Methods

rnf :: InstructionOrData -> () #

Generic InstructionOrData Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Associated Types

type Rep InstructionOrData 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep InstructionOrData = D1 ('MetaData "InstructionOrData" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "Data" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Instruction" 'PrefixI 'False) (U1 :: Type -> Type))
Show InstructionOrData Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

Eq InstructionOrData Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type BitSize InstructionOrData Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep InstructionOrData Source # 
Instance details

Defined in Protocols.Experimental.Axi4.Common

type Rep InstructionOrData = D1 ('MetaData "InstructionOrData" "Protocols.Experimental.Axi4.Common" "clash-protocols-0.1-inplace" 'False) (C1 ('MetaCons "Data" 'PrefixI 'False) (U1 :: Type -> Type) :+: C1 ('MetaCons "Instruction" 'PrefixI 'False) (U1 :: Type -> Type))